import re
from argparse import ArgumentParser
from .doc_handler import DocumentHandler
from .translator import Translator
from .transposer import Transposer

'''
Function that translates the input numbers file into the specified key.
'''
def translate(inputFile, key, outputFile):
    dh = DocumentHandler(inputFile)
    tr = Translator(dh.document, key)
    output = tr.parse()
    dh.save(outputFile)

'''
Function that transposes the input chord file into the specified key.
'''
def transpose(inputFile, origKey, newKey, outputFile):
    dh = DocumentHandler(inputFile)
    tr = Transposer(dh.document, origKey, newKey)
    tr.parse()
    dh.save(outputFile)

def getOutputName(inputFile, key):
    inputFile = re.sub(r"\s+\([A-G][#b]?\)", '', inputFile)
    dotPos = inputFile.index('.')
    name = inputFile[:dotPos]
    extension = inputFile[dotPos:]

    return "{} ({}){}".format(name, key, extension)

def main():
    parser = ArgumentParser()
    parser.add_argument('-t', '--transpose', help='transpose from key to key', nargs=3)
    parser.add_argument('--translate', help='translate from number system to chords', nargs=2)
    args = parser.parse_args()

    if args.transpose is not None:
        inputFile, key1, key2 = args.transpose
        transpose(inputFile, key1, key2, getOutputName(inputFile, key2))
    elif args.translate is not None:
        inputFile, key = args.translate
        translate(inputFile, key, getOutputName(inputFile, key))
    else:
        print("Invalid command: For help, enter chord123 --help")



import re
from .music_theory import KEYS_SHARP, MODE, INCREMENT, hasChords, isMusical

class Translator:
    def __init__(self, document=None, key='A'):
        self.document = document
        self.key = key

    def numToChord(self, chordNum, keyId, doMode):
        # convert chord to 0-based index
        j = int(chordNum[0])-1

        # get correct chord
        chordId = keyId+INCREMENT[j]
        # check for sharps or flats
        if '#' in chordNum:
            chordId += 1
        elif 'b' in chordNum:
            chordId -= 1
        if not doMode:
            return KEYS_SHARP[chordId%12]

        # check if mode explicitly defined
        if 'M' in chordNum:
            return KEYS_SHARP[chordId%12]
        elif 'm' in chordNum:
            return KEYS_SHARP[chordId%12]+'m'

        # compute manually
        if '#' in chordNum or 'b' in chordNum:
            return KEYS_SHARP[chordId%12]

        return "{}{}".format(KEYS_SHARP[chordId%12], MODE[j])

    def translate(self, number, keyId):
        numbers = number.split('/');
        chordNum = numbers[0]

        chord = self.numToChord(chordNum, keyId, True)
        if len(numbers) == 2:
            bassNum = numbers[1]
            if bassNum == '':
                return "{}/".format(chord)
            else:
                return "{}/{}".format(chord, self.numToChord(bassNum, keyId, False))
        return chord

    def parse(self):
        origKeyId = KEYS_SHARP.index(self.key)
        keyId = origKeyId

        for p in self.document.paragraphs:
            inline = p.runs
            for i in range(len(inline)):
                if '+' in inline[i].text:
                    [transposeBy] = re.findall(r"([0-9]+)", inline[i].text)
                    keyId = (origKeyId+int(transposeBy))%12
                else:
                    inline[i].text = re.sub(r"([0-9](?!x)[^\s]*)", lambda number: self.translate(number.group(0), keyId), inline[i].text)
